package plugins;

import java.awt.*;
import pluginSDK.*;
import main.*;

/**
 * Title:        the Wondrous Farm<br>
 * Description:  Un comportement de suivi de leader<br>
 * Copyright:    Copyright (c) 2002<br>
 * Company:      minfo<br>
 * @author Guillaume Vaillant
 * @version 1
 */
 public class LeaderFollowing extends Behaviours {

 	private Boids leader;
	private double old;
	private boolean follow;
	private static double leadershipRange;

 	static {
                System.out.println("chargement du plugin de comportement: \"Suivi de leader\"...");
		leadershipRange = 0.0;
        }

	public LeaderFollowing() { 
		follow = false;
		leader = null;
	}

	public String getBehaviour() { return "Seeker"; }

	public boolean actsOn(Boids b, Entity e, Flock f) {

		if (leadershipRange == 0.0)
			leadershipRange = 1.5 * f.minDist;

		if (leader == null) {
			leader = lookForNewLeader(f);
			follow = false;
		}

		if ((leader != null) && ((! leader.isLeader()) || (! f.contains(leader)))) {
			System.out.println("[" + b.getName() + "]: No more leader");
			leader = null;
			follow = false;
			return false;
		}

		if ((leader != null) && (b.equals(leader))) // si le leader fait partie du
			return false;			    // troupeau, il ne peut pas se
							    // suivre.
		
		if ((leader != null) && (! follow)) {
			double nHdg = b.directionVersPoint(leader.getPosX(), leader.getPosY());
               	 	double d = b.distanceDepuisUnPoint(leader.getPosX(), leader.getPosY());

			b.changeDir(nHdg);
                	b.move(b.getAccel());

			if (f.minDist < d && d <= leadershipRange) {
				System.out.println("[" + b.getName() + "]: I'm following [" + leader.getName() + "]");
				b.changeDir(leader.getHeading());
				old = b.getSpeed();
				b.setSpeed(leader.getSpeed());
				follow = true;
				b.extraStuff();
			}
		} else if (leader != null) { // donc si on suit

			double d = b.distanceDepuisUnPoint(leader.getPosX(), leader.getPosY());
			if (f.minDist < d && d <= leadershipRange) {
				b.changeDir(leader.getHeading());
				b.move();
			} else {
				System.out.println("[" + b.getName() + "] I lose my leader");
				b.changeDir((b.getHeading() + ((Math.random() < 0.5) ? 0.05 : -0.05)) % (2 * Math.PI));
				b.setSpeed(old);
				b.move(b.getAccel());
				follow = false;
			}
		} else 
			return false; // on a pas de leader
		return true;
	}

	/**
	 * On considre que 'b' suit le leader s'il est  une distance respectable
	 * de lui, qu'il est dans son champ d'attraction (et qu'il a le mme cap).
	 */
	private boolean isBehind(Boids b, boolean f, double m) {
	
		double h = leader.getHeading() % (2 * Math.PI);
		double hb = b.getHeading() % (2 * Math.PI);
		int xe = leader.getPosX(), ye = leader.getPosY();
		int xb = b.getPosX(), yb = b.getPosY();
		double d = b.distanceDepuisUnPoint(xe, ye);
		int c1 = Math.max(leader.getHeight(), leader.getWidth());
		int c2 = Math.max(b.getHeight(), b.getWidth());
		int c = Math.max(c1, c2);
		boolean okD = ((m + c) < d) && (d <= (leadershipRange + c)); // on est  bonne distance
		boolean okH = (h == hb);

		if (! okD) // si on est pas  bonne distance, de toute manire, c'est foutu
			return false;

		if ((0 <= h) && (h < (Math.PI / 2.0)))
			if (f) 
				return ((xb >= xe) && (yb <= ye) && okD && okH);
			else
				return ((xb >= xe) && (yb <= ye) && okD);
		if (((Math.PI / 2.0) <= h) && (h < Math.PI))
			if (f) 
				return ((xb <= xe) && (yb <= ye) && okD && okH);
			else
				return ((xb <= xe) && (yb <= ye) && okD);
		if ((Math.PI <= h) && (h < (3.0 * Math.PI / 2.0)))
			if (f)
				return ((xb <= xe) && (yb >= ye) && okD && okH);
			else
				return ((xb <= xe) && (yb >= ye) && okD);
		if (((3.0 * Math.PI / 2.0) <= h) && (h < (2.0 * Math.PI)))
			if (f)
				return ((xb >= xe) && (yb >= ye) && okD && okH);
			else
				return ((xb >= xe) && (yb >= ye) && okD);
		System.out.println("returning");
		return false; // peu de chance, pour faire plaisir  javac
	}

	private Boids lookForNewLeader(Flock f) {
		Entity chief = null;
		int i = 0;

		while (i < (f.population() - 1)) {
			chief = f.getEntity(i);
			if (chief.getType() == Entity.IS_BOID)
				if (! ((Boids)chief).isLeader())
					chief = null;
				else
					return (Boids)chief;
			i++;
		}

		return null;
	}	

}
