package plugins;

import pluginSDK.*;
import main.*;

/**
 * Title:        the Wondrous Farm
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:      minfo
 * @author Guillaume VAILLANT
 * @version 0.8
 */
public class obstacleAvoidance extends Behaviours {


        static {
                System.out.println("chargement du plugin de comportement: \"vitement d'Obstacle\"...");
        }

        public obstacleAvoidance() {}

        public String getBehaviour() {
                return "obstacleAvoidance";
        }

        public boolean actsOn(Boids p, Entity t, Flock f) {

		double d = (t != null) ? p.distanceDepuisUnPoint(t.getPosX(), t.getPosY()) : 0.0;

                if (t != null && 
		    ((t.getType() == Entity.IS_BOID) || (t.getType() == Entity.IS_OBSTACLE)) &&
		    detecteCollision(p, t, f) && (d < f.minDist)) {
			double nHdg = avoid(p, t);
			p.changeDir(nHdg);
			p.move();
			return true;
                }
		return false;
        }

	/**
	 * Dtection de la collision: si au prochain point que 'b' atteint il
	 * se trouve dans 'e', alors il y a collision.
	 * @param b: le boid qui pourrait percuter
	 * @param e: l'entit qui pourrait tre percute
	 * @return boolean: 'true' si collision, 'false' sinon
	 */
        public static boolean detecteCollision(Boids b, Entity e, Flock f) {
		
		int nextX = (b.getPosX() + (int)(Math.cos(b.getHeading()) * b.getSpeed())) % f.getWidth();
		int nextY = (b.getPosY() + (int)(Math.sin(b.getHeading()) * b.getSpeed())) % f.getHeight();

		int xe1 = e.getPosX() - e.getWidth() / 2;
		int xe2 = e.getPosX() + e.getWidth() / 2;
		int ye1 = e.getPosY() - e.getHeight() / 2;
		int ye2 = e.getPosY() + e.getHeight() / 2;

		int xb1 = nextX - b.getWidth() / 2;
		int xb2 = nextX + b.getWidth() / 2;
		int yb1 = nextY - b.getHeight() / 2;
		int yb2 = nextY + b.getHeight() / 2;

		if ((nextX >= xe1 && nextX <= xe2 && nextY >= ye1 && nextY <= ye2) ||
		    (xb1 >= xe1 && xb1 <= xe2 && yb1 >= ye1 && yb1 <= ye2) ||
		    (xb2 >= xe1 && xb2 <= xe2 && yb1 >= ye1 && yb1 <= ye2) ||
		    (xb1 >= xe1 && xb1 <= xe2 && yb2 >= ye1 && yb2 <= ye2) ||
		    (xb2 >= xe1 && xb2 <= xe2 && yb2 >= ye1 && yb2 <= ye2))
			return true;

		return false;
        }

	private double avoid(Boids b, Entity e) {

		int xe = e.getPosX();
		int ye = e.getPosY(); 
		int xe1 = xe - e.getWidth() / 2;
		int xe2 = xe + e.getWidth() / 2;
		int ye1 = ye - e.getHeight() / 2;
		int ye2 = ye + e.getHeight() / 2;
	
		int xb = b.getPosX();
		int yb = b.getPosY();
		int d = Math.max(b.getHeight(), b.getWidth());

		double heading = 0.0;
		
		if (xb < xe1 && yb <= ye)
			heading = b.directionVersPoint(xe1 - d, ye1 - d);
		if (xb < xe1 && yb > ye)
			heading = b.directionVersPoint(xe1 - d, ye2 + d);

		if (xb > xe2 && yb <= ye)
			heading = b.directionVersPoint(xe2 + d, ye1 - d);
		if (xb > xe2 && yb > ye)
			heading = b.directionVersPoint(xe2 + d, ye2 + d);

		if (yb < ye1 && xb <= xe)
			heading = b.directionVersPoint(xe1 - d, ye1 - d);
		if (yb < ye1 && xb > xe)
			heading = b.directionVersPoint(xe2 + d, ye1 - d);

		if (yb > ye2 && xb <= xe)
			heading = b.directionVersPoint(xe1 - d, ye2 + d);
		if (yb > ye2 && xb > xe)
			heading = b.directionVersPoint(xe2 + d, ye2 + d);

		return heading;
	}
}
