package classLoader;

import java.util.*;
import java.net.*;
import java.io.*;
import pluginSDK.*;

/**
 * Title:        the Wondrous Farm<br>
 * Description: Le chargeur de classes pour les plugins du projet.<br>
 *		Son utilisation ne se limite bien sur pas au cadre du projet, il
 *		pourrait presque faire le caf si on le lui demandait :-).<br>
 *		Il n'impose qu'une seule limitation sur le l'emplacement des classes
 *		 charger: tre sur le disque local. La classe n'a mme pas besoin de
 *		se trouver dans le <code>CLASSPATH</code>.<br>
 * Copyright:    Copyright (c) 2002<br>
 * Company:      minfo<br>
 * @author Guillaume VAILLANT
 * @version 0.1
 */
public class MyClassLoader extends ClassLoader {

        private String location;
        private Class clazz;
        private URL url;
	private Package pluginsPackage = null; // est-ce rellement util?

	/**
	 * Le constructeur, le paramtre de localisation des classes pass en paramtre
	 * doit reprsenter l'URL du rpertoire o se trouve les classe  charger.<br>
	 * Si les classes se trouvent dans <code>toto/MesPlugins</code>, l'URL sera
	 * <code>file:toto/MesPlugins</code>.
	 * @param loc: l'URL du rpertoire o se trouvent les classes
	 * @param c: mettez <code>null</code>, il ne sert  rien
	 * @param p: le <i>package</i> auquel appartiennent les classes  charger.<br>
	 * Il ne doit pas tre le mot-clef <code>null</code>.
	 */
        public MyClassLoader(String loc, Class c, Package p) {
		if (p == null) {
			try {
				p = definePackage("plugins", "plugins", "", "", "plugins", "", "", null);
				pluginsPackage = p;
			} catch (Exception e) {
				GUI.Main3.displayError(this.getClass(), "MyClassLoader() (I)", e);
			}
		} 
                location = new String(loc);
		try {
                        url = new URL(location);
                } catch (Exception e) {
			GUI.Main3.displayError(this.getClass(), "MyClassLoader() (II)", e);
                }
                clazz = c;
        }

	/**
	 * Renvoie l'URL du rpertoire o se trouvent les classes  charger
	 * @return java.net.URL
	 */
	public URL getURL() { return url; }

	/**
	 * @deprecated ne PLUS JAMAIS utiliser, ne sert plus  rien, devrait tre efface.
	 */
	public ArrayList loadPlugins() {
    		URL u = this.getResource(location);

    		if (u == null) {
      			System.err.println("[ERR] Rpertoire de plugins specifi n'est pas correct " + url);
      			return new ArrayList(); // ou null ??
    		}

      		return loadPluginsClassFromURL(u, location, clazz);
        }

	/**
	 * Charge les plugins depuis le rpertoire spcifi au constructeur.
	 * @return java.util.ArrayList: la liste des classes/plugins chargs.
	 */
        public ArrayList loadPlugins2() {

                if (url == null)
                        System.err.println("[ERR] Can't load classes, probably bad location: " + location);
                else
                        return loadPluginsClassFromURL(url, "plugins", clazz);

		return null;
        }

	/**
	 * Charge les plugins depuis le rpertoire spcifi au constructeur.
	 * @return java.util.ArrayList: la liste des classes/plugins chargs.
	 */
        private ArrayList loadPluginsClassFromURL(URL url, String rep, Class clazz) {
    		// Contiendra les plugins cherchs
    		ArrayList plugins = new ArrayList();

    		File dir = new File(url.getFile());

    		if (dir == null || !dir.isDirectory()) {
                        System.err.println("[ERR] malformed URL: <<" + url + ">>\n[---] Can't load plugins.");
      			return plugins;
    		}
                System.out.println("[INFO] loading plugins from: <<" + url + ">>");

    		// on rcupre la liste des fichiers du rpertoire
    		String[] list = dir.list();

    		// On trie pour que les plugins apparaissent dans le menu
    		// par ordre alphabtique
    		Arrays.sort(list);

		byte[] classData = null;

    		for (int i = 0; i < list.length; i++) {
      			// Un plugin ne peut tre une classe interne
      			if (list[i].indexOf('$') == -1 && list[i].endsWith(".class")) {
				// le fichier .class avec son chemin complet:
				String classWithPath = url + "/" + list[i];
				// les donnes de la classe
				classData = loadClassDataAsByte(classWithPath);
				// le nom de la classe: "plugins.nomClasse"
				String className = rep + "." + list[i].substring(0,list[i].indexOf('.'));
				// maintenant on cre le plugin,...
				Plugin plugin = instanciateClass(classData, className);

				if (plugin != null)
	    				plugins.add(plugin);
      			}
    		}

    		return plugins;
  	}

	/**
	 * Rcupre le bytecode de la classe passe en paramtre.
	 * @param clName: le nom de la classe avec son chemin complet sous forme d'URL (http://www.xyz.com/plugins/Toto.class, ...)
	 * @return byte[]: le bytecode.
	 */
	private byte[] loadClassDataAsByte(String clName) {
		try {
			URL classURL = new URL(clName); // on rcupre l'URL complte de la classe
			URLConnection connection = classURL.openConnection(); // on se connecte dessus
			InputStream classStream = connection.getInputStream(); // on prpare dequoi lire son contenu
			int len = connection.getContentLength(); // on rcupre la taille du contenu
			byte[] data = (len < 0) ? null : new byte[len]; // si len == -1 alors PROBLEME, sinon on alloue le buffer
			classStream.read(data); // on lit les octets de la classe et on les stocke ds le buffer
			classStream.close(); 
			return data;
		} catch (Exception e) {
			GUI.Main3.displayError(this.getClass(), "loadClassAsByte()", e);
		}
		return null;
	}

	/**
	 * Instancie la classe  partir de son bytecode et de son nom.
	 * @param data: le bytecode de la classe
	 * @param name: le nom complet et qualifi de la classe (plugins.MegaBoid, ...)
	 * @return pluginSDK.Plugin: une instance de <code>Plugin</code> correspondant  <code>name</code>
	 */
	private Plugin instanciateClass(byte[] data, String name) {
		// on vrifie qu'on a bien des donnes pour la classe:
		if (data == null)
			return null;
			
		// on cre la classe (normalement, plus de pb de CLASSPATH comme avant:)
		Class newClass = this.defineClass(name, data, 0, data.length);
		if (newClass == null) {
			System.out.println(">>> ClassFormatError!!! <<<");
			throw new ClassFormatError();
		}

		try { // pour lier la classe avec les autres
			resolveClass(newClass);
		} catch (Exception le) {
			GUI.Main3.displayError(this.getClass(), "instanciateClass()", le);
		}
			
		// on essaye d'instancier le plugin:	
		Plugin p = null;
		try {
			p = (Plugin)newClass.newInstance();
		} catch (Exception e) {
			GUI.Main3.displayError(this.getClass(), "instanciateClass()", e);
		}
		return p;
	}

}
