package main;

import java.awt.*;
import java.util.*;
import pluginSDK.*;
import javax.swing.*;

/**
 * Titre :        The Wondrous Farm<br>
 * Description :  Le monde dans lequel voluent les <code>Boids</code><br>
 * Copyright :    Copyright (c) 2002<br>
 * Socit :      Maitrise d'Info<br>
 */
public class Flock extends JPanel implements Runnable,  java.io.Serializable {

        private ArrayList world;
	private int nbMaxPets,   	// nb max de "boids"
		    width,       	// largeur du panel
		    height;      	// hauteur du panel
	public double minDist = 40.0;   // distance minimale entre deux "boids", en pixels
	private Thread worldThread;
        private ListIterator wLI;
        private int nbBoids, nbObstacles, nbReset;
        private boolean pause;
	private boolean startSim;

        /**
         * Cre un nouveau Flock/Monde pour la simulation.
         * @param w: largeur du monde
         * @param h: hauteur du monde
         */
	public Flock(int w, int h) { // dimensions panel + nb boids
		width = w;
		height = h;
		nbMaxPets = 0;
                world = new ArrayList();
                this.setSize(width, height);
                this.setBackground(Color.white);
		worldThread = new Thread(this);
		worldThread.start();
                nbBoids = nbObstacles = nbReset = 0;
                pause = false;
		startSim = false;
	}

        /**
         * @return int: le nombre d'entit (Obstacle et Boids) prsent dans ce Flock
         */
        public int population() { return world.size(); }

        /**
	 * Renvoie toutes les entits prsentes dans le monde
	 * @return ArrayList: la liste des entits du monde.
	 */
	public ArrayList getPopulation() { return world; }

        /**
         * @return int: la largeur du Flock
         */
        public int getWidth() { return width; }

        /**
         * @return int: la hauteur du Flock
         */
        public int getHeight() { return height; }

        /**
         * Retire une entit du monde.
         * @param e: le Boid ou l'Obstacle  liminer.
         */
        public void remove(Entity e) {
                int i = world.indexOf(e);
                if (e.getType() == Entity.IS_BOID) {
			((Boids)e).isLeader(false);
                        nbBoids--;
		}
                if (e.getType() == Entity.IS_OBSTACLE)
                        nbObstacles--;
                world.remove(i);
        }

        /**
         * Rajoute une entit dans le Flock.
         * @param e: le Boid ou l'Obstacle  ajouter.
         */
        public void add(Entity e) {
                if (e.getType() == Entity.IS_BOID)
                        nbBoids++;
                if (e.getType() == Entity.IS_OBSTACLE)
                        nbObstacles++;
                world.add(e);
        }

        /**
         * Rcupre la i_me entit dans le Flock.
         * @param i: l'indice compris entre <code>0</code> et <code>this.population() - 1</code>
         * @return Entity: le Boid ou l'Obstacle correspondant.
         */
        public Entity getEntity(int i) {
                return (Entity)world.get(i);
        }

	/**
	 * Teste l'appartenance de l'entit au Flock
	 * @param e: l'entit  tester
	 * @return boolean: <code>true</code> si appartenance confirme, <code>false</code> sinon
	 */
	public boolean contains(Entity e) {
		return world.contains(e);
	}

        /**
         * Rinitialise le Flock, on retire toutes les entits du monde.
         */
        public void empty() {
		pause();
		for(int i = 0; i < world.size(); i++) 
			remove((Entity)world.get(i));
                world.clear();
                nbReset++;
                nbBoids = nbObstacles = 0;
		pause();
		startSim = false;
        }

        /**
         * La mthode qui dirige le monde.<br>
         * Elle active ou dsactive toutes les entits du Flock.
         */
        public void run() {
                int i = 0;
                while (true) {
			if (startSim && (! pause)) {
	                        while (i < world.size()) {
	                                Entity e = (Entity)world.get(i++);
	                                if (e.getType() == Entity.IS_BOID)
        	                                ((Boids)e).doYourStuff(this);
                	        }
                        	i = 0;
			
	                        try {
	                                worldThread.sleep(5);
	                        } catch (Exception e) {
	                                System.err.println("[ERR] Exception caught: " + e.getMessage());
	                                e.printStackTrace();
        	                }
			}
                        repaint();
                }
        }

        /**
         * Affiche tous les lements du Flock
         * @param g: le "contexte graphique" o l'on dessine.
         */
        public void paintComponent(Graphics g) {
		super.paintComponent(g);
                for(int i = 0; i < world.size(); i++)
                        ((Entity)world.get(i)).drawEntity(g);
        }

        /**
         * Renvoie une description de l'tat courrant du Flock.
         * @return String: la chaine qui dcrit cet tat.
         */
        public String getFlockStats() {
                String stats = "Number of \"Entity\": ";
                stats += (population() + "\n");
                stats += "Number of \"Boids\": ";
                stats += (nbBoids + "\n");
                stats += "Number of \"Obstacle\": ";
                stats += (nbObstacles + "\n");
                stats += "\nYou reset the simulation ";
                stats += (nbReset + " times.\n\n");
                return stats;
        }

        /**
         * (d)Mettre en pause la simulation<br>
	 * Tout est bloqu, on ne peut rien modifier dans la simulation
         */
        public void pause() {
                pause = (! pause);
                if (pause) {
                        System.out.println("[INFO] Entering suspend mode");
                } else {
                        System.out.println("[INFO] Leaving suspend mode");
                }
        }

	/**
	 * Dmarrer ou stopper la simulation courrante<br>
	 * On peut rajouter/supprimer ou changer l'tat des entits du Flock.
	 */
	public void startSim() {
		startSim = (! startSim);
	}

        /**
         * Description Sommaire du Flock
	 * @return String: la description
         */
        public String toString() {
                String desc = this.getClass().getName() + ", " + width + "x" + height + ", " + nbMaxPets;

                return desc;
        }

}
