package pluginSDK;

import java.awt.*;
import java.awt.geom.Point2D;
import java.util.*;
import main.*;

/**
 * Titre :        The Wondrous Farm
 * Description :  La classe de base des plugins de Bestioles/Boids
 * Copyright :    Copyright (c) 2002
 * Socit :      Maitrise d'Info
 */
public abstract class Boids extends Entity {

        protected double speed,         // vitesse du boid
                         heading,       // direction du boid in radian: [0..2Pi], 0rad <=> North
                         acceleration,  // acclration du boid
                         viewAngle;     // champ de vision du boid
        private String name;            // son nom (prince Charles, Usama, ...)
        private int type = IS_BOID;
        private Behaviours b;
        private ArrayList listB;
        private int nbKill;
        private int worldW, worldH;


        public int getWorldWidth() { return worldW; }
        public int getWorldHeigth() { return worldH; }
        public void setWorldWidth(int w) { worldW = w; }
        public void setWorldHeigth(int h) { worldH = h; }

	public Boids() {
		super();
		speed = 0.0; heading = 0.0; acceleration = 0.0; viewAngle = 1.0; 
                nbKill = 0;
                listB = new ArrayList();
                worldW = worldH = 0;
	}

	/**
	 * @param X: la coordonne horizontale du Boid
	 * @param Y: la coordonne verticale du Boid
	 * @param spd: la vitesse du Boid
	 * @param hdg: son cap en radian, compris entre 0 et 2*Pi
	 * @param accel: son acclration
	 * @param va: son champ de vision en radian, compris entre 0 et 2*Pi
	 * @param mn: le nom du Boid (Toto, Lady Die, Prince Charles, ...)
	 * @param f: les Bestioles qui gravitent autour
	 * @param w: la largeur du Boid
	 * @param h: sa hauteur.
	 */
        public Boids(int X, int Y,
                      double spd, double hdg, double accel,
                      double va, String nm, ArrayList f, int w, int h) {
                super(X, Y, f);
                speed = spd;
                heading = hdg;
                acceleration = accel;
                viewAngle = va;
                name = new String(nm);
                nbKill = 0;
                listB = new ArrayList();
                worldW = w; worldH = h;
        }

	/**
	 * Renvoie le type du Boid (@see pluginSDK.Plugin)
	 */
        public int getType() { return IS_BOID; }

	/**
	 * Renvoie vrai si on peut voir l'entit (boid ou obstacle)
	 * @param p: l'entit que l'on pourrait voir
	 * @return boolean: <code>true</code> si on la voit, <code>false</code> sinon
	 */
        public boolean canSee(Entity p) {

	        double teta = directionVersPoint(p.getPosX(), p.getPosY()); // <== modifi par seb
                double gamma1 = (heading - (viewAngle / 2.0)) % Math.PI;
                double gamma2 = (heading + (viewAngle / 2.0)) % Math.PI;

                if ((gamma2 >= teta) && (teta <= gamma1))
                        return true;
                else
                        return false;
        }

	/**
	 * modifier le cap du boid
	 * @param hdg: le nouveau cap en radian, entre 0 et 2*Pi
	 */
        public void changeDir(double hdg) { // change pet's heading
                heading = hdg;
        }

	/**
	 * idem
	 */
        public void setHeading(double hdg) { heading = hdg; }
	
	/**
	 * Renvoie le cap du Boid
	 * @return double: le cap en radians
	 */
        public double getHeading() { return heading; }

	/**
	 * Faire avancer le Boid.
	 */
        public void move() {
                x += (int)(Math.cos(heading) * (speed <= 0 ? 1.0 : speed ));
                y += (int)(Math.sin(heading) * (speed <= 0 ? 1.0 : speed ));
                x %= worldW;
                y %= worldH;
                if(x < 0)
                        x += worldW;
                if(y < 0)
                        y += worldH;
        }
	
	/**
	 * Faire avancer le Boid encore plus vite
	 */
        public void move(double accel) {
                x += (int)(Math.cos(heading) *
			   (speed <= 0 ? 1.0 : speed ) *
			   (accel <= 0 ? 1.0 : accel));
                y += (int)(Math.sin(heading) *
			   (speed <= 0 ? 1.0 : speed ) *
			   (accel <= 0 ? 1 : accel));
                x %= worldW;
                y %= worldH;
                if(x < 0)
                        x += worldW;
                if(y < 0)
                        y += worldH;
        }

        public double getSpeed() { return speed; }
        public void setSpeed(double s) { speed = s; }

        public double getAccel() { return acceleration; }
        public void setAccel(double a) { acceleration = a; }

	/**
	 * Renvoie l'angle du champ de vision du Boid
	 * @return double: l'angle en radians
	 */
        public double getViewAngle() { return viewAngle; }
	/**
	 * Modifie le champ de vision du Boid
	 * @param a: l'angle en radians entre 0 et 2*Pi
	 */
        public void setViewAngle(double a) { viewAngle = a; }

	/**
	 * Renvoie le type du Boid
	 * @return String: le type du Boid (par ex: "Boids")
	 */
	public abstract String getKind();

        public String getName() { return name; }
        public void setName(String n) { name = new String(n); }

	/**
	 * Renvoie le nombre de victimes du Boid.
	 * @return int: un entier >= 0
	 */
        public int getNbKill() { return nbKill; }
	/**
	 * Augmente le nombre de victimes du Boid de 1
	 */
        public void incNbKill() { nbKill++; }

	/**
	 * Rajoute un cmoportement au Boid
	 * @param b: le nouveau comportement
	 * @see pluginSDK.Behaviours
	 */
        public void addBehaviour(Behaviours b) { listB.add(b); }
	/**
	 * Renvoie les comportements de ce Boid.
	 * @return java.util.ArrayList: la liste des comportements.
	 */
        public ArrayList getBehaviours() { return listB; }

	/**
	 * Fait agir le Boid en fonction des comportements dans le monde
	 * auquel il appartient.<br>
	 * Tous ses comportements sont appliqus jusqu' ce qu'il y en ait
	 * un qui modifie sont tat.
	 * @param f: le monde du Boid
	 * @see: main.Flock
	 */
        public void doYourStuff(Flock f) {
		boolean hasChanged = false;
                Entity p = seekNearest();
                for(int i = 0; i < listB.size(); i++)
			if (! hasChanged)
                        	hasChanged = ((Behaviours)listB.get(i)).actsOn(this, p, f);
        }

        //====== ajout par seb
	/**
	 * Renvoie la direction de ce Boid par rapport  un point
	 * @param bx: l'abscisse du point
	 * @param by: l'ordonne du point
	 * @return double: l'angle en radian
	 */
        public double directionVersPoint(int bx, int by) {
	        double d = Point2D.distance(x,y,bx,by);
	        if (d>0.0) {
	                double dircos = Math.acos(((double)(bx-x))/d);
	                double dirsin = Math.asin(((double)(by-y))/d);
	                if (dirsin>=0)
                                return  dircos;
                        else
                                return -dircos;
	        }
	        else
                        return 0;
        }

        /**
         * Des actions supplmentaire (qd le Boid meure, tue qq'un, ...)<br>
	 * Cette mthode est optionnelle.
         */
        public void extraStuff() {}

	/**
	 * Dessine le Boid.
	 * @see java.awt.Graphics
	 */
        public abstract void drawEntity(Graphics g);

}



