package pluginSDK;

import java.awt.*;
import java.util.*;
import main.*;

/**
 * Title:        the Wondrous Farm<br>
 * Description: La classe correspondant  toutes les entits que l'on trouve dans la simulation,
 *              Boids, Obstacles, ...<br>
 * Copyright:    Copyright (c) 2002<br>
 * Company:      minfo
 */
public abstract class Entity implements Plugin {

        protected int x, y, w, h; // coordonnes, largeur & hauteur
        protected ArrayList myWorld;
        protected Color myColor;
        private int type = IS_MOTHER;
        private boolean isLeader, isSelected;
        private String path;

        // qq constantes:
        public static final int IS_MOTHER = 100;
        public static final int IS_BOID = 101;
        public static final int IS_OBSTACLE = 102;
        public static final int IS_BEHAVIOUR = 103;

        public Entity() {
                x = 0; y = 0;
                myWorld = null;
                myColor = new Color((float)Math.random(), (float)Math.random(), (float)Math.random());
                isLeader = false;
                isSelected = false;
        }

	/**
	 * @param X: abscisse de l'entit dans le monde
	 * @param Y: son ordonne
	 * @param f: ses copines.
	 */
        public Entity(int X, int Y, ArrayList f) {
                x = X; y = Y;
                myWorld = f;
                myColor = new Color((float)Math.random(), (float)Math.random(), (float)Math.random());
                isLeader = false;
                isSelected = false;
        }

        public int getType() { return IS_MOTHER; }

        public int getPosX() { return x; }
        public void setPosX(int X) { x = X; }

        public int getPosY() { return y; }
        public void setPosY(int Y) { y = Y; }

        public void setHeight(int h) { this.h = h; }
        public int getHeight() { return h; }

        public void setWidth(int w) { this.w = w; }
        public int getWidth() { return w; }

        public String getBehaviour() { return null; }

        public Color whatIsMyColor() { return myColor; }
        public void setMyColor(Color c) { myColor = c; }

        public ArrayList getWorld() { return myWorld; }
        public void setWorld(ArrayList f) { myWorld = f; }

        public boolean isLeader() { return isLeader; }
        public void isLeader(boolean l) { isLeader = l; }

        public boolean getSelected() { return isSelected; }
        public void setSelected(boolean s) { isSelected = s; }

        public void setPath(String p) {
                path = new String(p);
        }
        public String getPath() {
                return path;
        }

        public abstract String getKind();

        public String toString() {
                String s = this.getClass().getName();
                return s.substring(s.indexOf(".") + 1, s.length());
        }

        public abstract void drawEntity(Graphics g);

        public double computeTargetHeading(int X, int Y) {
                // -PI <= alpha <= PI
                double alpha;
                if (x != X)
                        alpha = Math.atan((Y - y) / (X - x)) % (2 * Math.PI);
                else
                        alpha = (y < Y) ? -(Math.PI / 2) : (Math.PI / 2);

                if (alpha >= 0) // )<= alpha <= PI
                        return (2 * Math.PI - alpha);
                else // -PI <= alpha < 0
                        return (alpha - 2 * Math.PI);
        }

        public double directionDepuisUnPoint (int xOrigine, int yOrigine, double axe) {
        // Rends l'angle en radians (entre ]-PI, PI[) de la bbte courante
        // par rapport  un point origine et  l'axe donn
        // tan x = cot oppos / cot adjacent
                double angle;
                if (x != xOrigine)
                        angle = Math.atan((y - yOrigine) / (x - xOrigine));
                else if (y < yOrigine)
                        angle = - Math.PI / 2;
                else
                        angle = Math.PI / 2;

                // atan a deux solutions, donc il faut corriger...
                if (x < xOrigine) {
                        if (y < yOrigine)
                                angle -= Math.PI;
                        else
                                angle += Math.PI;
                }

                // Il suffit maintenant de soustraire l'axe  la direction (en radians)
                // et obtenir une direction entre -PI et PI
                double dir = angle - axe;

                if (dir >= Math.PI)
                        return dir - Math.PI * 2;
                else if (dir < -Math.PI)
                        return dir + Math.PI * 2;
                else
                        return dir;
        }

        public double distanceDepuisUnPoint(int x1, int y1){
                // rend la distance entre la bbte et un point donn
                return Math.sqrt(Math.pow(x1 - x, 2) + Math.pow(y1 - y, 2));
        }

        public Entity seekNearest() {
                if (myWorld.size() <= 0)
                        return null;
                double dist = 100000.0;
                int i = 0;
                Entity nearest = null;
                while(i < myWorld.size()) {
                        Entity e = (Entity)myWorld.get(i);
                        if (! this.equals(e)) {
                                double d = e.distanceDepuisUnPoint(x, y);
                                if (d < dist) {
                                        nearest = e;
                                        dist = d;
                                }
                        }
                        i++;
                }

                return nearest;
        }

        public static Entity seekNearest(Flock f, int x, int y) {

                if (f.population() <= 0) // probablement difficile d'obtenir une valeur <0, mais bon...
                        return null;

                double dist = 100000.0;
                int i = 0;
                Entity nearest = null;
                while(i < f.population()) {
                        Entity e = f.getEntity(i);
                        if (! (e.x == x && e.y == y)) {
                                double d = e.distanceDepuisUnPoint(x, y);
                                if (d < dist) {
                                        nearest = e;
                                        dist = d;
                                }
                        }
                        i++;
                }

                return nearest;
        }

}
