package utils;

import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JPopupMenu;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import javax.swing.JOptionPane;

import main.*;
import pluginSDK.*;
import GUI.EntityCreator;

/**
 * Title:        the Wondrous Farm<br>
 * Description:  L'couteur de souris qui nous permet de slectionner des <code>Entity</code>
 * dans le monde de la simulation.<br>
 * Copyright:    Copyright (c) 2002<br>
 * Company:      minfo<br>
 */
public class EntitySelector implements MouseListener {

        private Flock world;
        private Entity selectedEntity;
        private JPopupMenu entityMenu;
        private JMenuItem stats, leader, kill, toto, behav;
        private EntityCreator c;

        /**
         * @param f: le monde (<code>Flock</code>) dans lequel on cherche 
         * effectuer des oprations  la souris (slection + affichage PopUp)
         */
        public EntitySelector(Flock f, EntityCreator c) {
                world = f;
                this.c = c;
                selectedEntity = null;
                entityMenu = new JPopupMenu();

                toto = new JMenuItem("");
                toto.setEnabled(false);
                entityMenu.add(toto);

                entityMenu.add(new JSeparator());

                stats = new JMenuItem("informations");
                stats.setToolTipText("show entity descriptions");
                stats.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                popupStats(e);
                        }
                });
                entityMenu.add(stats);

                leader = new JMenuItem("leadership");
                leader.setToolTipText("set leadership of entity");
                leader.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                entityMenu.setVisible(false);
                                if (selectedEntity != null)
                                        ((Boids)selectedEntity).isLeader(! ((Boids)selectedEntity).isLeader());
                                entityMenu.setVisible(false);
                                selectedEntity.setSelected(false);
                        }
                });
                entityMenu.add(leader);

                kill = new JMenuItem("kill");
                kill.setToolTipText("kill selected entity");
                kill.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                selectedEntity.setSelected(false);
                                world.remove(selectedEntity);
                                entityMenu.setVisible(false);
                        }
                });
                entityMenu.add(kill);
        }

        // on ne considre que les click de souris avec les boutons 1 & 3
        public void mouseClicked(MouseEvent e) {
                int msBtn = e.getModifiers(); // le bouton de la souris

                switch(msBtn) {
                case MouseEvent.BUTTON1_MASK: // slection d'une entit
                        if (selectedEntity != null) // pas de slection multiple
                                selectedEntity.setSelected(false);
                        selectedEntity = Entity.seekNearest(world, e.getX(), e.getY());
                        if (selectedEntity != null) {
                                selectedEntity.setSelected(true);
                        }
                        break;
                case MouseEvent.BUTTON3_MASK: // menu de l'entit
                        if (selectedEntity != null) {
                                showEntityPopup(selectedEntity, e.getX(), e.getY());
                        } else
                                System.err.println("[WNG] No entity selected");
                        break;
                default:
                        System.err.println("[WNG] unknown mouse button clicked\n" +
                                           "[---] or no actions are linked to this button");
                }

        }

        // Les mthodes suivantes concernent des vnements qui ne nous interressent oas.
        public void mousePressed(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}

        // Mthodes auxiliaires:
        /**
         * Pour afficher les stats du Boid, le dfinir comme leader,
         * le tuer, ...
         */
        private void showEntityPopup(Entity e, int x, int y) {
                int type = e.getType();
                String titre = null;

                if (type != Entity.IS_BOID) {
                        titre = new String(e.getKind());
                        leader.setEnabled(false);
                } else {
                        titre = new String(((Boids)e).getName());
                        leader.setEnabled(true);
                }

                toto.setText(titre);
                entityMenu.setLocation(x, y);
                entityMenu.setVisible(true);
        }

        /**
         * Cette mthode affiche les statitistiques de l'entit slectionne:
         * nombre de kills, nombre de collisions, nom, type, "leadership",
         * troupeau.
         */
        private void popupStats(ActionEvent e) {
                PluginDescriptor desc = new PluginDescriptor(selectedEntity);
                entityMenu.setVisible(false);

                String mesg = desc.getDesc();
                JOptionPane.showInternalMessageDialog(world, mesg, "Infos. about: " + selectedEntity.getKind(),
                                                      JOptionPane.INFORMATION_MESSAGE);
                c.toBack();
                selectedEntity.setSelected(false);
        }
}
