package utils;


import java.io.*;
import main.*;
import pluginSDK.*;

/**
 * Title:        the Wondrous Farm<br>
 * Description:	 Cette classe permet de charger des plugins depuis un fichier correspondant
 *		  une simulation sauvegarde ou d'crire des plugins dans un fichier pour
 *		 une sauvegarde.<br>
 * Copyright:    Copyright (c) 2002<br>
 * Company:      minfo<br>
 */
public class PluginReaderWriter {

        private File pluginFile;
        private Flock w;
        private char mode;

	/**
	 * Le constructeur, trois paramtres:
	 * @param f: le fichier sur lequel on va travailler (pas ouvert)
	 * @param m: le mode de travail: <code>'r'</code> pour lire, <code>'w'</code> pour crire.
	 * @param w: le <code>Flock</code> ou monde sur lequel on travaille.
	 */
	public PluginReaderWriter(File f, char m, Flock w) {
                pluginFile = f;
                this.w = w;
                if (m == 'r' || m == 'w')
                        mode = m;
        }

	/**
	 * La mthode magique :-)<br>
	 * Pour une lecture, si le monde n'tait pas vide, il est vid, puis remplit avec
	 * celui sauvegard.
	 * @return main.Flock: <code>null</code> si on sauvait la simulation, sinon le monde qui
	 *		       tait sauvegard dans le fichier.
	 */
	public Flock DO() {
                if (mode == 'w') {
                        writeFlockToFile();
                        return null;
                }
                return createFlockFromFile();
        }

	/**
	 * Ecrit le monde dans un fichier.
	 */
        private void writeFlockToFile() {
                String s = null;
                PluginDescriptor d = null;
                ObjectOutputStream oos = null;
                try {
                        oos = new ObjectOutputStream(
                                new FileOutputStream(pluginFile));
                        for(int i = 0; i < w.population(); i++)
                                oos.writeObject(w.getEntity(i));
                        oos.close();
                } catch (Exception e) {
			GUI.Main3.displayError(this.getClass(), "writeFlockToFile()", e);
                }
        }

	/**
	 * Charge le monde depuis un fichier.<br>
	 * Rinitialise le monde s'il n'tait pas vierge.
	 */
        private Flock createFlockFromFile() {
                String s = null;
                String tmp = null;
                ObjectInputStream ois = null;
                Entity entity = null;
                try {
                        ois = new ObjectInputStream(
                                new FileInputStream(pluginFile));
                        w.empty();
			entity = (Entity)ois.readObject();
                        if (entity != null)
                        	w.add(entity);
                        while (entity != null) {
                                entity = (Entity)ois.readObject();
                                if (entity != null)
                                        w.add(entity);
                        }
                } catch (Exception e) {
			GUI.Main3.displayError(this.getClass(), "createFlockFromFile()", e);
                }
                return w;
        }

}
